
/* 
 * Get duration in hours.
 */

double GetDuration(TChain * chain) 
{
    // branches
    ULong64_t tms;
    chain->SetBranchAddress("tms", &tms);

    // find tmin and tmax
    int shifts = 0;
    double shift = 0;
    double Tp = 0, TFirst = 0, TLast = 0;    
    for (int i=0; i<chain->GetEntries(); i++) 
    {
        chain->GetEntry(i);
        double T = tms;
        if (Tp > T) {
            shift += Tp;
            shifts++;
        }
        if (i == 0) TFirst = T + shift;
        if (i == chain->GetEntries() - 1) TLast = T + shift;
        Tp = T;
    }

    double seconds = (TLast - TFirst) * 1E-3;
    double minutes = seconds / 60.;
    double hours = minutes / 60.;
    
    chain->ResetBranchAddresses();

    return hours;
}

/* 
 * Locate and fit 1.6-MeV gamma peak
 */

void FitGammaPeak(TH1D * h0, double& counts, double& e0, double& sigma, double& dcounts, double& de0, double& dsigma) 
{
    // TSpectrum
    int maxPeaks = 1;
    TSpectrum * tspec = new TSpectrum(maxPeaks);

    // make a copy 
    TH1D * h = (TH1D*)h0->Clone("hcopy");
    h->GetXaxis()->SetRangeUser(1350, 1700);
    
    // get bin width
    double binWidth = h->GetBinWidth(1);

    // estimate background    
    TH1D * hbackgr = (TH1D*)tspec->Background(h);
    
    // search for 1.6-MeV peak
    tspec->Search(h);
    double * xPos = tspec->GetPositionX();
    double * yPos = tspec->GetPositionX();

    // subtract background    
    TH1D * hsub = (TH1D*)h->Clone("hsub");
    TH1D * hinv = (TH1D*)hbackgr->Clone("hinv");
    hinv->Scale(-1);
//***    hsub->Add(hinv);

    // fit function
    TF1 * fitf = new TF1("fitf", "[0]*TMath::Gaus(x,[1],[2],true)+[3]", 0, 1E4);

    // fit range
    double fitmin = xPos[0] - 50;
    double fitmax = xPos[0] + 50;

    // start values
    fitf->SetParameter(0, yPos[0]);
    fitf->SetParameter(1, xPos[0]);
    fitf->SetParameter(2, 20);
    fitf->SetParameter(3, 0);

    // fit    
    hsub->Fit("fitf", "EBQ", "", fitmin, fitmax);
    fitmin = fitf->GetParameter(1) - 3*fitf->GetParameter(2);
    fitmax = fitf->GetParameter(1) + 3*fitf->GetParameter(2);
    hsub->Fit("fitf", "ES+BQ", "", fitmin, fitmax);

    // get fitted values
    counts  = fitf->GetParameter(0) / binWidth;
    dcounts = fitf->GetParError(0) / binWidth;
    e0      = fitf->GetParameter(1);
    de0     = fitf->GetParError(1);
    sigma   = fitf->GetParameter(2);
    dsigma  = fitf->GetParError(2);
}


/* 
 * Plot transmission as a function of magnetic-field setting.
 */

void TransmissionExp(bool veto = false, double emin = 100, double emax = 5400)
{
    // directories
    TString expdir = "/home/oliskir/Desktop/f20/data/sorted/new/";
    TString simdir = "070617/";

    // gamma detection efficiency
    const double gammaEff  = 0.54E-4;
    const double dgammaEff = 0.10E-4;

    // beta spectrum binning
    const int bins = (emax - emin) / 10;
    
    // number of magnetic field settings
    const int N = 10;
    vector<int> bmax = {15};//{15, 20, 25, 30, 35, 40, 44, 46, 48, 50};

    // runs at each setting
    vector<int> runs[N];
    runs[0] = {366};
    runs[1] = {332};
    runs[2] = {367};
    runs[3] = {333};
    runs[4] = {368};
    runs[5] = {334};
    runs[6] = {369};
    runs[7] = {360};
    runs[8] = {361, 362, 363, 364, 365, 370, 371, 372, 373, 375, 376, 379, 380};
    runs[9] = {358};

    // background run (337)
    int rBackgr = 337;
    TChain * cBackgr = new TChain("data");    
    cBackgr->Add(expdir+Form("*0%i*", rBackgr));    
    TH1F * hBackgr0 = new TH1F("hBackgr0", "hBackgr0", bins, emin, emax);
    cBackgr->Draw("E0 >> hBackgr0", "Z0 != 10");
    double hoursBackgr = GetDuration(cBackgr);
    
    cout << endl;
    cout << " Background run:" << endl;
    cout << Form(" * Run:      %i", rBackgr) << endl;
    cout << Form(" * Entries:  %lli", cBackgr->GetEntries()) << endl;
    cout << Form(" * Duration: %.1f hours", hoursBackgr) << endl;
    cout << endl;

    // canvas
    TCanvas * c1 = new TCanvas("c1");

    // loop over magnetic field settings
    int i = 0;
    for (auto& b : bmax) 
    {
        // add files
        TChain * cExp = new TChain("data");    
        for (auto& r : runs[i]) {
            TString fname = expdir + Form("*0%i*", r);
            cExp->Add(fname);    
        }
        
        // duration of experimental run
        double hours = GetDuration(cExp);

        // integrate background spectrum
        TH1D * hBackgr = (TH1D*)hBackgr0->Clone("hBackgr");
        int binx1      = hBackgr->GetXaxis()->FindBin(emin);
        int binx2      = hBackgr->GetXaxis()->FindBin(emax);
        double backgr  = hBackgr->Integral(binx1, binx2);
        double dbackgr = sqrt(backgr);
        
        // scale 
        backgr  *= hours/hoursBackgr;
        dbackgr *= hours/hoursBackgr;
        hBackgr->Scale(hours/hoursBackgr);

        // singles histograms
        TString nb = Form("hb%i", i);        
        TH1D * hb  = new TH1D(nb, nb, bins, emin, emax);
        TString ng = Form("hg%i", i);        
        TH1D * hg  = new TH1D(ng, ng, 1000, 0, 3000);

        // draw
        cExp->Draw("E0 >> " + nb,  "M0==1 && Z0!=10");
        cExp->Draw("E2 >> " + ng,  "M2==1 && Z2!=10");

        // integrate beta spectrum
        double betas  = hb->Integral(binx1, binx2);
        double dbetas = sqrt(betas);

        // subtract background
        betas -= backgr;
        dbetas = sqrt(pow(dbetas, 2) + pow(dbackgr, 2));
        
        // integrate 1.6-MeV peak in singles spectrum
        double gammas, dgammas, centroid, dcentroid, sigma, dsigma;
        FitGammaPeak(hg, gammas, centroid, sigma, dgammas, dcentroid, dsigma);

        // coincidence histogram
        TString nbg = Form("hbg%i", i);        
        TH1F * hbg  = new TH1F(nbg, nbg, 1000, 0, 3000);
        
        // draw
        double egmin = centroid - 3*sigma;
        double egmax = centroid + 3*sigma;
        cExp->Draw("E0 >> " + nbg, Form("M0==1 && M2==1 && Z0 != 10 && Z2 != 10 && E2 >= %f && E2 <= %f", egmin, egmax));

        // integrate coincidence beta spectrum
        double coinc  = hbg->Integral(binx1, binx2);
        double dcoinc = sqrt(coinc);

        // simulated beta spectrum
        TString fname = simdir + Form("b%i.root", b);
        TFile * fSim = new TFile(fname, "READ");
        TTree * tSim = (TTree*)fSim->Get("Detector");

        // histogram 
        TString nbSim = Form("hbSim%i", i);        
        TH1D * hbSim  = new TH1D(nbSim, nbSim, bins, emin, emax);

        // draw
        tSim->Draw("EDepSignal >> " + nbSim,  "");

        // integrate
        double betasSim  = hbSim->Integral(binx1, binx2);
        double dbetasSim = sqrt(betasSim);

        // scaling factor
        double decays = gammas / gammaEff;
        double norm   = ((TParameter<double>*)(tSim->GetUserInfo()->FindObject("NORMALIZATION")))->GetVal(); 
        double x = decays / norm;
        
        // scale spectrum and integral
        hbSim->Scale(x);        
        double relUncert = sqrt(pow(dbetasSim/betasSim, 2) + pow(dgammas/gammas, 2) + pow(dgammaEff/gammaEff, 2));
        betasSim *= x;
        dbetasSim = betasSim * relUncert;
        
        cout << Form(" Bmax:      %.2f T", 0.01*b) << endl;
        cout << Form(" Duration:  %.2f hours", hours) << endl; 
        cout << Form(" Backgr:    %.2E (%.1f%%)", backgr, dbackgr/backgr*100.) << endl; 
        cout << Form(" Betas:     %.2E (%.1f%%)", betas, dbetas/betas*100.) << endl; 
        cout << Form(" Gammas:    %.0f (%.1f%%)", gammas, dgammas/gammas*100.) << endl; 
        cout << Form(" Coinc:     %.0f (%.1f%%)", coinc, dcoinc/coinc*100.) << endl; 
        cout << Form(" Sim betas: %.2E (%.1f%%)", betasSim, dbetasSim/betasSim*100.) << endl; 
    }
}
